==> edge-2.0.0-sources.jar: ph/com/nightowlstudios/ApplicationVerticle.java <==
package ph.com.nightowlstudios;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.Handler;
import io.vertx.core.Promise;
import io.vertx.core.http.HttpHeaders;
import io.vertx.core.http.HttpMethod;
import io.vertx.core.http.HttpServer;
import io.vertx.core.http.HttpServerOptions;
import io.vertx.core.json.JsonObject;
import io.vertx.core.net.JksOptions;
import io.vertx.core.net.PemKeyCertOptions;
import io.vertx.core.net.PfxOptions;
import io.vertx.ext.auth.jwt.JWTAuth;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.BodyHandler;
import io.vertx.ext.web.handler.CorsHandler;
import io.vertx.ext.web.handler.ErrorHandler;
import io.vertx.ext.web.handler.LoggerHandler;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author <a href="mailto:josephharveyangeles@gmail.com">Joseph Harvey Angeles - <i>@yev</i></a>
 * @since 7/14/20
 */
public abstract class ApplicationVerticle<T extends ApplicationConfig> extends AbstractVerticle {

    private final ApplicationConfig appConfig;

    protected ApplicationVerticle (ApplicationConfig appConfig) {
        this.appConfig = appConfig;
    }

    @Override
    public void start(Promise<Void> startPromise) {
        Router rootRouter = Router.router(vertx);
        onRootRouterCreate(rootRouter);
        rootRouter.route().handler(
                CorsHandler
                        .create(appConfig.getAllowedOrigins())
                        .allowCredentials(allowCORSCredentials())
                        .allowedMethods(getAllowedMethods())
                        .allowedHeaders(getAllowedHeaders())
                        .exposedHeaders(getExposedHeaders())
        );
        rootRouter.route().handler(BodyHandler.create());
        rootRouter.mountSubRouter(appConfig.getApiPrefix(), buildApiRouter());

        createHttpServer()
                .requestHandler(rootRouter)
                .listen(appConfig.getPort(), http -> {
                    if (http.succeeded()) {
                        startPromise.complete();
                        log.info(appConfig.getBannerText());
                        log.info("{} online at PORT: {}", appConfig.getApplicationName(), http.result().actualPort());
                        onServerDeployed(http.result());
                        return;
                    }
                    log.error("Error running server: " + http.cause());
                    startPromise.fail(http.cause());
                    onServerDeployFail(http.cause());
                });
    }

    /**
     * Called immediately after Router creation.
     *
     * @param router the root router
     */
    protected void onRootRouterCreate(Router router){}

    /**
     * Called before mounting API subrouter.
     *
     * @param router api subrouter
     */
    protected void beforeAPIRouterMount(Router router){}

    protected void onServerDeployed(HttpServer httpServer){}
    protected void onServerDeployFail(Throwable error){}

    protected boolean allowCORSCredentials() { return true; }
    protected Set<HttpMethod> getAllowedMethods() { return  ALLOWED_METHODS; }
    protected Set<String> getAllowedHeaders() { return ALLOWED_HEADERS; }
    protected Set<String> getExposedHeaders() { return EXPOSED_HEADERS; }

    protected Handler<RoutingContext> createRouteLogHandler() {
        return LoggerHandler.create();
    }
    protected Handler<RoutingContext> createRouteFailureHandler() {
        return ErrorHandler.create();
    }

    protected abstract JWTAuth createAuthProvider();
    protected abstract Class<?>[] getResourceClasses();

    private HttpServer createHttpServer() {
        if (appConfig.isProduction()) {
            return vertx.createHttpServer(createSSLHttpOptions());
        }
        return vertx.createHttpServer();
    }

    private HttpServerOptions createSSLHttpOptions () {
        JsonObject ssl = appConfig.getSSLConfig();

        HttpServerOptions options = new HttpServerOptions()
                .setUseAlpn(false)
                .setSsl(true);

        switch (ssl.getString("type", StringUtils.EMPTY)) {
            case "jks":
                options.setKeyStoreOptions(
                        new JksOptions()
                                .setPath(ssl.getString("path"))
                                .setPassword(ssl.getString("password"))
                );
                break;
            case "pfx":
                options.setPfxKeyCertOptions(
                        new PfxOptions()
                                .setPath(ssl.getString("path"))
                                .setPassword(ssl.getString("password"))
                );
                break;
            case "pem":
                options.setPemKeyCertOptions(
                        new PemKeyCertOptions()
                                .setKeyPath(ssl.getString("keyPath"))
                                .setCertPath(ssl.getString("certPath"))
                );
            default:
                log.error("Missing SSL Configuration");
                throw new RuntimeException("No SSL Configuration Provided");
        }
        return options;
    }

    private Router buildApiRouter() throws RuntimeException {
        Router router = Router.router(vertx);
        beforeAPIRouterMount(router);
        router.route().handler(createRouteLogHandler());
        router.route().failureHandler(createRouteFailureHandler());

        JWTAuth authProvider = createAuthProvider();
        for (Class<?> controller : getResourceClasses()) {
            try {
                controller
                        .getDeclaredConstructor(Router.class, JWTAuth.class)
                        .newInstance(router, authProvider);
            } catch (Exception e) {
                log.error("Unable to load {}. Error on {}. ", controller.getCanonicalName(), e.getMessage());
                e.printStackTrace();
            }
        }
        return router;
    }

    private static final Logger log = LoggerFactory.getLogger(ApplicationVerticle.class);

    private static final Set<HttpMethod> ALLOWED_METHODS = new HashSet<>(
            Arrays.asList(
                    HttpMethod.GET,
                    HttpMethod.PUT,
                    HttpMethod.POST,
                    HttpMethod.PATCH,
                    HttpMethod.DELETE,
                    HttpMethod.OPTIONS
            )
    );

    private static final Set<String> ALLOWED_HEADERS = new HashSet<>(
            Arrays.asList(
                    HttpHeaders.CONTENT_TYPE.toString(),
                    HttpHeaders.AUTHORIZATION.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_ORIGIN.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_HEADERS.toString(),
                    HttpHeaders.ACCESS_CONTROL_REQUEST_METHOD.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_CREDENTIALS.toString()
            )
    );

    private static final Set<String> EXPOSED_HEADERS = new HashSet<>(Collections.singletonList(HttpHeaders.AUTHORIZATION.toString()));

    static final String BANNER_TXT =
            "▓█████ ▓█████▄   ▄████ ▓█████    \n" +
            "▓█   ▀ ▒██▀ ██▌ ██▒ ▀█▒▓█   ▀    \n" +
            "▒███   ░██   █▌▒██░▄▄▄░▒███      \n" +
            "▒▓█  ▄ ░▓█▄   ▌░▓█  ██▓▒▓█  ▄    \n" +
            "░▒████▒░▒████▓ ░▒▓███▀▒░▒████▒   \n" +
            "░░ ▒░ ░ ▒▒▓  ▒  ░▒   ▒ ░░ ▒░ ░   \n" +
            " ░ ░  ░ ░ ▒  ▒   ░   ░  ░ ░  ░   \n" +
            "   ░    ░ ░  ░ ░ ░   ░    ░      \n" +
            "   ░  ░   ░          ░    ░  ░   \n" +
            "        ░                        \n";
}

==> edge-2.0.1-sources.jar: ph/com/nightowlstudios/ApplicationVerticle.java <==
package ph.com.nightowlstudios;

import io.vertx.core.AbstractVerticle;
import io.vertx.core.Handler;
import io.vertx.core.Promise;
import io.vertx.core.http.HttpHeaders;
import io.vertx.core.http.HttpMethod;
import io.vertx.core.http.HttpServer;
import io.vertx.core.http.HttpServerOptions;
import io.vertx.core.json.JsonObject;
import io.vertx.core.net.JksOptions;
import io.vertx.core.net.PemKeyCertOptions;
import io.vertx.core.net.PfxOptions;
import io.vertx.ext.auth.jwt.JWTAuth;
import io.vertx.ext.web.Router;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.handler.BodyHandler;
import io.vertx.ext.web.handler.CorsHandler;
import io.vertx.ext.web.handler.ErrorHandler;
import io.vertx.ext.web.handler.LoggerHandler;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author <a href="mailto:josephharveyangeles@gmail.com">Joseph Harvey Angeles - <i>@yev</i></a>
 * @since 7/14/20
 */
public abstract class ApplicationVerticle<T extends ApplicationConfig> extends AbstractVerticle {

    private final T appConfig;

    protected ApplicationVerticle (T appConfig) {
        this.appConfig = appConfig;
    }

    @Override
    public void start(Promise<Void> startPromise) {
        Router rootRouter = Router.router(vertx);
        onRootRouterCreate(rootRouter);
        rootRouter.route().handler(
                CorsHandler
                        .create(appConfig.getAllowedOrigins())
                        .allowCredentials(allowCORSCredentials())
                        .allowedMethods(getAllowedMethods())
                        .allowedHeaders(getAllowedHeaders())
                        .exposedHeaders(getExposedHeaders())
        );
        rootRouter.route().handler(BodyHandler.create());
        rootRouter.mountSubRouter(appConfig.getApiPrefix(), buildApiRouter());

        createHttpServer()
                .requestHandler(rootRouter)
                .listen(appConfig.getPort(), http -> {
                    if (http.succeeded()) {
                        startPromise.complete();
                        log.info(appConfig.getBannerText());
                        log.info("{} online at PORT: {}", appConfig.getApplicationName(), http.result().actualPort());
                        onServerDeployed(http.result());
                        return;
                    }
                    log.error("Error running server: " + http.cause());
                    startPromise.fail(http.cause());
                    onServerDeployFail(http.cause());
                });
    }

    protected T getAppConfig() { return this.appConfig; }

    /**
     * Called immediately after Router creation.
     *
     * @param router the root router
     */
    protected void onRootRouterCreate(Router router){}

    /**
     * Called before mounting API subrouter.
     *
     * @param router api subrouter
     */
    protected void beforeAPIRouterMount(Router router){}

    protected void onServerDeployed(HttpServer httpServer){}
    protected void onServerDeployFail(Throwable error){}

    protected boolean allowCORSCredentials() { return true; }
    protected Set<HttpMethod> getAllowedMethods() { return  ALLOWED_METHODS; }
    protected Set<String> getAllowedHeaders() { return ALLOWED_HEADERS; }
    protected Set<String> getExposedHeaders() { return EXPOSED_HEADERS; }

    protected Handler<RoutingContext> createRouteLogHandler() {
        return LoggerHandler.create();
    }
    protected Handler<RoutingContext> createRouteFailureHandler() {
        return ErrorHandler.create();
    }

    protected abstract JWTAuth createAuthProvider();
    protected abstract Class<?>[] getResourceClasses();

    private HttpServer createHttpServer() {
        if (appConfig.isProduction()) {
            return vertx.createHttpServer(createSSLHttpOptions());
        }
        return vertx.createHttpServer();
    }

    private HttpServerOptions createSSLHttpOptions () {
        JsonObject ssl = appConfig.getSSLConfig();

        HttpServerOptions options = new HttpServerOptions()
                .setUseAlpn(false)
                .setSsl(true);

        switch (ssl.getString("type", StringUtils.EMPTY)) {
            case "jks":
                options.setKeyStoreOptions(
                        new JksOptions()
                                .setPath(ssl.getString("path"))
                                .setPassword(ssl.getString("password"))
                );
                break;
            case "pfx":
                options.setPfxKeyCertOptions(
                        new PfxOptions()
                                .setPath(ssl.getString("path"))
                                .setPassword(ssl.getString("password"))
                );
                break;
            case "pem":
                options.setPemKeyCertOptions(
                        new PemKeyCertOptions()
                                .setKeyPath(ssl.getString("keyPath"))
                                .setCertPath(ssl.getString("certPath"))
                );
            default:
                log.error("Missing SSL Configuration");
                throw new RuntimeException("No SSL Configuration Provided");
        }
        return options;
    }

    private Router buildApiRouter() throws RuntimeException {
        Router router = Router.router(vertx);
        beforeAPIRouterMount(router);
        router.route().handler(createRouteLogHandler());
        router.route().failureHandler(createRouteFailureHandler());

        JWTAuth authProvider = createAuthProvider();
        for (Class<?> controller : getResourceClasses()) {
            try {
                controller
                        .getDeclaredConstructor(Router.class, JWTAuth.class)
                        .newInstance(router, authProvider);
            } catch (Exception e) {
                log.error("Unable to load {}. Error on {}. ", controller.getCanonicalName(), e.getMessage());
                e.printStackTrace();
            }
        }
        return router;
    }

    private static final Logger log = LoggerFactory.getLogger(ApplicationVerticle.class);

    private static final Set<HttpMethod> ALLOWED_METHODS = new HashSet<>(
            Arrays.asList(
                    HttpMethod.GET,
                    HttpMethod.PUT,
                    HttpMethod.POST,
                    HttpMethod.PATCH,
                    HttpMethod.DELETE,
                    HttpMethod.OPTIONS
            )
    );

    private static final Set<String> ALLOWED_HEADERS = new HashSet<>(
            Arrays.asList(
                    HttpHeaders.CONTENT_TYPE.toString(),
                    HttpHeaders.AUTHORIZATION.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_ORIGIN.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_HEADERS.toString(),
                    HttpHeaders.ACCESS_CONTROL_REQUEST_METHOD.toString(),
                    HttpHeaders.ACCESS_CONTROL_ALLOW_CREDENTIALS.toString()
            )
    );

    private static final Set<String> EXPOSED_HEADERS = new HashSet<>(Collections.singletonList(HttpHeaders.AUTHORIZATION.toString()));

    static final String BANNER_TXT = "\n" +
            "▓█████ ▓█████▄   ▄████ ▓█████    \n" +
            "▓█   ▀ ▒██▀ ██▌ ██▒ ▀█▒▓█   ▀    \n" +
            "▒███   ░██   █▌▒██░▄▄▄░▒███      \n" +
            "▒▓█  ▄ ░▓█▄   ▌░▓█  ██▓▒▓█  ▄    \n" +
            "░▒████▒░▒████▓ ░▒▓███▀▒░▒████▒   \n" +
            "░░ ▒░ ░ ▒▒▓  ▒  ░▒   ▒ ░░ ▒░ ░   \n" +
            " ░ ░  ░ ░ ▒  ▒   ░   ░  ░ ░  ░   \n" +
            "   ░    ░ ░  ░ ░ ░   ░    ░      \n" +
            "   ░  ░   ░          ░    ░  ░   \n" +
            "        ░                        \n";
}
